#!/usr/bin/env python3
# -*- coding: utf-8; -*-

# Copyright 2020 University of Liège
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.




import numpy as np
import os
import shutil 

import Metafor.Mparams.TaskManager as tm
import Metafor.Mparams.mparams as mp


def main():
    if tm.rank == 0:
        #--------------------------------------------------------
        #                       Model parameters
        #--------------------------------------------------------
        model_parameters = {}
        isUnix = lambda: os.name == 'posix'

        if isUnix(): 
            model_parameters['Metafor_exe']  = '/home/kliegeois/Codes/Metafor/oo_metaB/bin/Metafor'
            model_parameters['Metafor_model'] = '/home/kliegeois/Codes/waves/Metafor/models/bord01'      
        else:     
            model_parameters['Metafor_exe']  = 'C:/Users/kimli_000/dev/oo_metaB/bin/Release/Metafor'
            model_parameters['Metafor_model'] = 'C:/Users/kimli_000/dev/waves/Metafor/models/bord01'
        #--------------------------------------------------------
        #                    Surrogate parameters
        #--------------------------------------------------------
        surrogate_parameters = {}
        surrogate_parameters['display'] = True 
        if not isUnix():
            surrogate_parameters['display'] = False 
        surrogate_parameters['normalisation'] = False 
        #--------------------------------------------------------    
        surrogate_parameters['fname'] = 'mparams.log'
        d = 2
        surrogate_parameters['d'] = d
        surrogate_parameters['P_name'] = np.array([ 'ih_H', 'sigEl0'])

        M = np.zeros((d,), dtype =np.int)
        for j in range(0,d):
            M[j] = 15    

        surrogate_parameters['M'] = M
            
        surrogate_parameters['Dom_min'] = np.array([ 1200., 200.])
        surrogate_parameters['Dom_max'] = np.array([ 1800., 650.])   

        N = np.zeros((d,), dtype =np.int)
        for j in range(0,d):
            N[j] = 2  

        surrogate_parameters['N'] = N
        #--------------------------------------------------------   
        #-------------------------------------------------------- 
        surrogate_parameters['function_type'] = 'monomial_1norm'
        surrogate_parameters['training_points'] = 'gamma'
        surrogate_parameters['solver'] = 'QR'
        surrogate_parameters['means'] = np.array([1495.,396.])
        surrogate_parameters['variances'] = np.array([1390.,660.])
        rho = -0.233 
        surrogate_parameters['correlations'] = np.array([[1.,rho],[rho,1.]])
        surrogate_parameters['f_mapping_method'] = 'Cholesky'
        surrogate_parameters['rho'] = rho  
        #--------------------------------------------------------
        surrogate_parameters['m1'] = 40
        surrogate_parameters['m2'] = 40
        #--------------------------------------------------------
        #                    Images parameters
        #--------------------------------------------------------
        images_parameters = {}
        images_parameters['xlabel'] = '$[$MPa$]$'
        images_parameters['ylabel'] = '$[$MPa$]$'
        images_parameters['zlabel'] = 'Angle $[$rad$]$'
        images_parameters['z_min'] = 0.04
        images_parameters['z_max'] = 0.07

    if mp.AvailableComputedResults():
        if tm.rank == 0:
            print("the master/sol.json file will be loaded")
            print("the master/Z.json file will be loaded")
            print("the master/W_sr.json file will be loaded")
            sol, Z, W_sr = mp.LoadComputedResults()

            if 1:
                from Metafor.Mparams.sequential import Z_evaluate
                Z1, W1_sr = Z_evaluate(model_parameters, surrogate_parameters)
                cond = np.zeros((14,1))
                for i in range(1,15):
                    N = np.zeros((d,), dtype =np.int)
                    for j in range(0,d):
                        N[j] = i  

                    surrogate_parameters['N'] = N
                    Z, W_sr = Z_evaluate(model_parameters, surrogate_parameters)

                    W_srZ = np.dot(W_sr,Z)
                    cond[i-1] = np.linalg.cond(W_srZ)
                print(cond)


            from Metafor.Mparams.sequential import scatterplot
            Image = scatterplot(sol, surrogate_parameters, images_parameters)
            from Metafor.Mparams.surrogate import surrogate_creation
            s_hat = surrogate_creation(sol, Z, W_sr, surrogate_parameters, Image, images_parameters)
    else:
        if tm.rank == 0:
            ##

            sol, Z, W_sr, s_hat = mp.master(model_parameters,surrogate_parameters,images_parameters)

            ##

            import codecs, json 

            tmp = sol.tolist() 
            file_path = "sol.json" 
            json.dump(tmp, codecs.open(file_path, 'w', encoding='utf-8'), separators=(',', ':'), sort_keys=True, indent=4) 

            tmp = Z.tolist() 
            file_path = "Z.json" 
            json.dump(tmp, codecs.open(file_path, 'w', encoding='utf-8'), separators=(',', ':'), sort_keys=True, indent=4) 

            tmp = W_sr.tolist() 
            file_path = "W_sr.json" 
            json.dump(tmp, codecs.open(file_path, 'w', encoding='utf-8'), separators=(',', ':'), sort_keys=True, indent=4) 

            tmp = s_hat.tolist() 
            file_path = "s_hat.json" 
            json.dump(tmp, codecs.open(file_path, 'w', encoding='utf-8'), separators=(',', ':'), sort_keys=True, indent=4) 

        else:
            mp.worker()

if __name__ == "__main__":
    main()
